@@ -86,18 +86,22 @@ static void irc_cmd_nick( irc_t *irc, char **cmd )
 	}
 	else if( irc->status & USTATUS_LOGGED_IN )
 	{
-		if( irc->status & USTATUS_IDENTIFIED )
+		/* WATCH OUT: iu from the first if reused here to check if the
+		   new nickname is the same (other than case, possibly). If it
+		   is, no need to reset identify-status. */
+		if( ( irc->status & USTATUS_IDENTIFIED ) && iu != irc->user )
 		{
 			irc_setpass( irc, NULL );
 			irc->status &= ~USTATUS_IDENTIFIED;
 			irc_umode_set( irc, "-R", 1 );
-			irc_usermsg( irc, "Changing nicks resets your identify status. "
+			irc_rootmsg( irc, "Changing nicks resets your identify status. "
 			             "Re-identify or register a new account if you want "
 			             "your configuration to be saved. See \x02help "
 			             "nick_changes\x02." );
 		}
 		
-		irc_user_set_nick( irc->user, cmd[1] );
+		if( strcmp( cmd[1], irc->user->nick ) != 0 )
+			irc_user_set_nick( irc->user, cmd[1] );
 	}
 	else
 	{
@@ -432,13 +436,13 @@ static void irc_cmd_oper_hack( irc_t *irc, char **cmd )
 			if( strcmp( a->pass, PASSWORD_PENDING ) == 0 )
 			{
 				set_setstr( &a->set, "password", password );
-				irc_usermsg( irc, "Password added to IM account "
-				             "%s(%s)", a->prpl->name, a->user );
+				irc_rootmsg( irc, "Password added to IM account "
+				             "%s", a->tag );
 				/* The IRC client may expect this. 491 suggests the OPER
 				   password was wrong, so the client won't expect a +o.
 				   It may however repeat the password prompt. We'll see. */
 				irc_send_num( irc, 491, ":Password added to IM account "
-				              "%s(%s)", a->prpl->name, a->user );
+				              "%s", a->tag );
 			}
 	}
 	else if( irc->status & OPER_HACK_IDENTIFY )
