@@ -34,11 +34,6 @@ irc_user_t *irc_user_new( irc_t *irc, const char *nick )
 	iu->nick = g_strdup( nick );
 	iu->user = iu->host = iu->fullname = iu->nick;
 	
-	if( set_getbool( &irc->b->set, "private" ) )
-		iu->last_channel = NULL;
-	else
-		iu->last_channel = irc->default_channel;
-	
 	iu->key = g_strdup( nick );
 	nick_lc( iu->key );
 	/* Using the hash table for speed and irc->users for easy iteration
@@ -156,6 +151,7 @@ int irc_user_set_nick( irc_user_t *iu, const char *new )
 	if( iu->host == NULL ) iu->host = g_strdup( iu->nick );
 	if( iu->fullname == NULL ) iu->fullname = g_strdup( iu->nick );
 	
+	g_free( iu->key );
 	iu->key = g_strdup( key );
 	g_hash_table_insert( irc->nick_user_hash, iu->key, iu );
 	irc->users = g_slist_insert_sorted( irc->users, iu, irc_user_cmp );
@@ -210,7 +206,11 @@ void irc_user_quit( irc_user_t *iu, const char *msg )
 		return;
 	
 	for( l = iu->irc->channels; l; l = l->next )
-		send_quit |= irc_channel_del_user( (irc_channel_t*) l->data, iu, IRC_CDU_SILENT, NULL );
+	{
+		irc_channel_t *ic = l->data;
+		send_quit |= irc_channel_del_user( ic, iu, IRC_CDU_SILENT, NULL ) &&
+		             ( ic->flags & IRC_CHANNEL_JOINED );
+	}
 	
 	if( send_quit )
 		irc_send_quit( iu, msg );
