@@ -37,64 +37,31 @@
 static char *oauth_sign( const char *method, const char *url,
                          const char *params, struct oauth_info *oi )
 {
-	sha1_state_t sha1;
 	uint8_t hash[sha1_hash_size];
-	uint8_t key[HMAC_BLOCK_SIZE+1];
+	GString *payload = g_string_new( "" );
+	char *key;
 	char *s;
-	int i;
 	
-	/* Create K. If our current key is >64 chars we have to hash it,
-	   otherwise just pad. */
-	memset( key, 0, HMAC_BLOCK_SIZE );
-	i = strlen( oi->sp->consumer_secret ) + 1 + ( oi->token_secret ? strlen( oi->token_secret ) : 0 );
-	if( i > HMAC_BLOCK_SIZE )
-	{
-		sha1_init( &sha1 );
-		sha1_append( &sha1, (uint8_t*) oi->sp->consumer_secret, strlen( oi->sp->consumer_secret ) );
-		sha1_append( &sha1, (uint8_t*) "&", 1 );
-		if( oi->token_secret )
-			sha1_append( &sha1, (uint8_t*) oi->token_secret, strlen( oi->token_secret ) );
-		sha1_finish( &sha1, key );
-	}
-	else
-	{
-		g_snprintf( (gchar*) key, HMAC_BLOCK_SIZE + 1, "%s&%s",
-		            oi->sp->consumer_secret, oi->token_secret ? oi->token_secret : "" );
-	}
+	key = g_strdup_printf( "%s&%s", oi->sp->consumer_secret, oi->token_secret ? oi->token_secret : "" );
 	
-	/* Inner part: H(K XOR 0x36, text) */
-	sha1_init( &sha1 );
-	
-	for( i = 0; i < HMAC_BLOCK_SIZE; i ++ )
-		key[i] ^= 0x36;
-	sha1_append( &sha1, key, HMAC_BLOCK_SIZE );
-	
-	/* OAuth: text = method&url&params, all http_encoded. */
-	sha1_append( &sha1, (const uint8_t*) method, strlen( method ) );
-	sha1_append( &sha1, (const uint8_t*) "&", 1 );
+	g_string_append_printf( payload, "%s&", method );
 	
 	s = g_new0( char, strlen( url ) * 3 + 1 );
 	strcpy( s, url );
 	http_encode( s );
-	sha1_append( &sha1, (const uint8_t*) s, strlen( s ) );
-	sha1_append( &sha1, (const uint8_t*) "&", 1 );
+	g_string_append_printf( payload, "%s&", s );
 	g_free( s );
 	
 	s = g_new0( char, strlen( params ) * 3 + 1 );
 	strcpy( s, params );
 	http_encode( s );
-	sha1_append( &sha1, (const uint8_t*) s, strlen( s ) );
+	g_string_append( payload, s );
 	g_free( s );
 	
-	sha1_finish( &sha1, hash );
+	sha1_hmac( key, 0, payload->str, 0, hash );
 	
-	/* Final result: H(K XOR 0x5C, inner stuff) */
-	sha1_init( &sha1 );
-	for( i = 0; i < HMAC_BLOCK_SIZE; i ++ )
-		key[i] ^= 0x36 ^ 0x5c;
-	sha1_append( &sha1, key, HMAC_BLOCK_SIZE );
-	sha1_append( &sha1, hash, sha1_hash_size );
-	sha1_finish( &sha1, hash );
+	g_free( key );
+	g_string_free( payload, TRUE );
 	
 	/* base64_encode + HTTP escape it (both consumers 
 	   need it that away) and we're done. */
@@ -107,16 +74,23 @@ static char *oauth_sign( const char *method, const char *url,
 
 static char *oauth_nonce()
 {
-	unsigned char bytes[9];
+	unsigned char bytes[21];
+	char *ret = g_new0( char, sizeof( bytes) / 3 * 4 + 1 );
 	
 	random_bytes( bytes, sizeof( bytes ) );
-	return base64_encode( bytes, sizeof( bytes ) );
+	base64_encode_real( bytes, sizeof( bytes), (unsigned char*) ret, "0123456789"
+	                    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0A" );
+	
+	return ret;
 }
 
 void oauth_params_add( GSList **params, const char *key, const char *value )
 {
 	char *item;
 	
+	if( !key || !value )
+		return;
+	
 	item = g_strdup_printf( "%s=%s", key, value );
 	*params = g_slist_insert_sorted( *params, item, (GCompareFunc) strcmp );
 }
@@ -126,6 +100,9 @@ void oauth_params_del( GSList **params, const char *key )
 	int key_len = strlen( key );
 	GSList *l, *n;
 	
+	if( params == NULL )
+		return;
+	
 	for( l = *params; l; l = n )
 	{
 		n = l->next;
@@ -150,6 +127,9 @@ const char *oauth_params_get( GSList **params, const char *key )
 	int key_len = strlen( key );
 	GSList *l;
 	
+	if( params == NULL )
+		return NULL;
+	
 	for( l = *params; l; l = l->next )
 	{
 		if( strncmp( (char*) l->data, key, key_len ) == 0 &&
@@ -160,7 +140,7 @@ const char *oauth_params_get( GSList **params, const char *key )
 	return NULL;
 }
 
-static void oauth_params_parse( GSList **params, char *in )
+void oauth_params_parse( GSList **params, char *in )
 {
 	char *amp, *eq, *s;
 	
@@ -228,6 +208,7 @@ void oauth_info_free( struct oauth_info *info )
 		g_free( info->request_token );
 		g_free( info->token );
 		g_free( info->token_secret );
+		oauth_params_free( &info->params );
 		g_free( info );
 	}
 }
@@ -280,6 +261,7 @@ static void *oauth_post_request( const char *url, GSList **params_, http_input_f
 	                     "Host: %s\r\n"
 	                     "Content-Type: application/x-www-form-urlencoded\r\n"
 	                     "Content-Length: %zd\r\n"
+	                     "Connection: close\r\n"
 	                     "\r\n"
 	                     "%s", url_p.file, url_p.host, strlen( post ), post );
 	g_free( post );
@@ -326,6 +308,7 @@ static void oauth_request_token_done( struct http_request *req )
 		st->auth_url = g_strdup_printf( "%s?%s", st->sp->url_authorize, req->reply_body );
 		oauth_params_parse( &params, req->reply_body );
 		st->request_token = g_strdup( oauth_params_get( &params, "oauth_token" ) );
+		st->token_secret = g_strdup( oauth_params_get( &params, "oauth_token_secret" ) );
 		oauth_params_free( &params );
 	}
 	
@@ -353,12 +336,10 @@ static void oauth_access_token_done( struct http_request *req )
 	
 	if( req->status_code == 200 )
 	{
-		GSList *params = NULL;
-		
-		oauth_params_parse( &params, req->reply_body );
-		st->token = g_strdup( oauth_params_get( &params, "oauth_token" ) );
-		st->token_secret = g_strdup( oauth_params_get( &params, "oauth_token_secret" ) );
-		oauth_params_free( &params );
+		oauth_params_parse( &st->params, req->reply_body );
+		st->token = g_strdup( oauth_params_get( &st->params, "oauth_token" ) );
+		g_free( st->token_secret );
+		st->token_secret = g_strdup( oauth_params_get( &st->params, "oauth_token_secret" ) );
 	}
 	
 	st->stage = OAUTH_ACCESS_TOKEN;
@@ -369,6 +350,7 @@ static void oauth_access_token_done( struct http_request *req )
 		st->auth_url = NULL;
 		g_free( st->request_token );
 		st->request_token = NULL;
+		oauth_params_free( &st->params );
 	}
 }
 
