@@ -63,7 +63,7 @@ account_t *account_add( bee_t *bee, struct prpl *prpl, char *user, char *pass )
 	s->flags |= ACC_SET_NOSAVE; /* Just for bw compatibility! */
 	
 	s = set_add( &a->set, "password", NULL, set_eval_account, a );
-	s->flags |= ACC_SET_NOSAVE | SET_NULL_OK;
+	s->flags |= ACC_SET_NOSAVE | SET_NULL_OK | SET_PASSWORD;
 	
 	s = set_add( &a->set, "tag", NULL, set_eval_account, a );
 	s->flags |= ACC_SET_NOSAVE;
@@ -338,6 +338,8 @@ void account_del( bee_t *bee, account_t *acc )
 		}
 }
 
+static gboolean account_on_timeout( gpointer d, gint fd, b_input_condition cond );
+
 void account_on( bee_t *bee, account_t *a )
 {
 	if( a->ic )
@@ -350,6 +352,9 @@ void account_on( bee_t *bee, account_t *a )
 	
 	a->reconnect = 0;
 	a->prpl->login( a );
+	
+	if( a->ic && !( a->ic->flags & OPT_SLOW_LOGIN ) )
+		a->ic->keepalive = b_timeout_add( 120000, account_on_timeout, a->ic );
 }
 
 void account_off( bee_t *bee, account_t *a )
@@ -363,6 +368,16 @@ void account_off( bee_t *bee, account_t *a )
 	}
 }
 
+static gboolean account_on_timeout( gpointer d, gint fd, b_input_condition cond )
+{
+	struct im_connection *ic = d;
+	
+	imcb_error( ic, "Connection timeout" );
+	imc_logout( ic, TRUE );
+	
+	return FALSE;
+}
+
 struct account_reconnect_delay
 {
 	int start;
