@@ -26,10 +26,12 @@
 
 static xt_status jabber_parse_roster( struct im_connection *ic, struct xt_node *node, struct xt_node *orig );
 static xt_status jabber_iq_display_vcard( struct im_connection *ic, struct xt_node *node, struct xt_node *orig );
+static int jabber_iq_disco_server( struct im_connection *ic );
 
 xt_status jabber_pkt_iq( struct xt_node *node, gpointer data )
 {
 	struct im_connection *ic = data;
+	struct jabber_data *jd = ic->proto_data;
 	struct xt_node *c, *reply = NULL;
 	char *type, *s;
 	int st, pack = 1;
@@ -50,7 +52,8 @@ xt_status jabber_pkt_iq( struct xt_node *node, gpointer data )
 	else if( strcmp( type, "get" ) == 0 )
 	{
 		if( !( ( c = xt_find_node( node->children, "query" ) ) ||
-		       ( c = xt_find_node( node->children, "ping" ) ) ) ||
+		       ( c = xt_find_node( node->children, "ping" ) ) ||
+		       ( c = xt_find_node( node->children, "time" ) ) ) ||
 		    !( s = xt_find_attr( c, "xmlns" ) ) )
 		{
 			/* Sigh. Who decided to suddenly invent new elements
@@ -68,7 +71,7 @@ xt_status jabber_pkt_iq( struct xt_node *node, gpointer data )
 			xt_add_child( reply, xt_new_node( "version", BITLBEE_VERSION, NULL ) );
 			xt_add_child( reply, xt_new_node( "os", ARCH, NULL ) );
 		}
-		else if( strcmp( s, XMLNS_TIME ) == 0 )
+		else if( strcmp( s, XMLNS_TIME_OLD ) == 0 )
 		{
 			time_t time_ep;
 			char buf[1024];
@@ -82,6 +85,31 @@ xt_status jabber_pkt_iq( struct xt_node *node, gpointer data )
 			strftime( buf, sizeof( buf ) - 1, "%Z", localtime( &time_ep ) );
 			xt_add_child( reply, xt_new_node( "tz", buf, NULL ) );
 		}
+		else if( strcmp( s, XMLNS_TIME ) == 0 )
+		{
+			time_t time_ep;
+			char buf[1024];
+			
+			buf[sizeof(buf)-1] = 0;
+			time_ep = time( NULL );
+			
+			xt_free_node( reply );
+			reply = xt_new_node( "time", NULL, NULL );
+			xt_add_attr( reply, "xmlns", XMLNS_TIME );
+			
+			strftime( buf, sizeof( buf ) - 1, "%Y%m%dT%H:%M:%SZ", gmtime( &time_ep ) );
+			xt_add_child( reply, xt_new_node( "utc", buf, NULL ) );
+			
+			strftime( buf, sizeof( buf ) - 1, "%z", localtime( &time_ep ) );
+			if( strlen( buf ) >= 5 )
+			{
+				buf[6] = '\0';
+				buf[5] = buf[4];
+				buf[4] = buf[3];
+				buf[3] = ':';
+			}
+			xt_add_child( reply, xt_new_node( "tzo", buf, NULL ) );
+		}
 		else if( strcmp( s, XMLNS_PING ) == 0 )
 		{
 			xt_free_node( reply );
@@ -94,6 +122,7 @@ xt_status jabber_pkt_iq( struct xt_node *node, gpointer data )
 		{
 			const char *features[] = { XMLNS_DISCO_INFO,
 			                           XMLNS_VERSION,
+			                           XMLNS_TIME_OLD,
 			                           XMLNS_TIME,
 			                           XMLNS_CHATSTATES,
 			                           XMLNS_MUC,
@@ -142,10 +171,10 @@ xt_status jabber_pkt_iq( struct xt_node *node, gpointer data )
 		/* This is a roster push. XMPP servers send this when someone
 		   was added to (or removed from) the buddy list. AFAIK they're
 		   sent even if we added this buddy in our own session. */
-			int bare_len = strlen( ic->acc->user );
+			int bare_len = strlen( jd->me );
 			
 			if( ( s = xt_find_attr( node, "from" ) ) == NULL ||
-			    ( strncmp( s, ic->acc->user, bare_len ) == 0 &&
+			    ( strncmp( s, jd->me, bare_len ) == 0 &&
 			      ( s[bare_len] == 0 || s[bare_len] == '/' ) ) )
 			{
 				jabber_parse_roster( ic, node, NULL );
@@ -301,6 +330,8 @@ static xt_status jabber_finish_iq_auth( struct im_connection *ic, struct xt_node
 		jd->flags |= JFLAG_AUTHENTICATED;
 		if( !jabber_get_roster( ic ) )
 			return XT_ABORT;
+		if( !jabber_iq_disco_server( ic ) )
+			return XT_ABORT;
 	}
 	
 	return XT_HANDLED;
@@ -315,8 +346,25 @@ xt_status jabber_pkt_bind_sess( struct im_connection *ic, struct xt_node *node,
 	if( node && ( c = xt_find_node( node->children, "bind" ) ) )
 	{
 		c = xt_find_node( c->children, "jid" );
-		if( c && c->text_len && ( s = strchr( c->text, '/' ) ) &&
-		    strcmp( s + 1, set_getstr( &ic->acc->set, "resource" ) ) != 0 )
+		if( !c || !c->text )
+		{
+			/* Server is crap, but this is no disaster. */
+		}
+		else if( strncmp( jd->me, c->text, strlen( jd->me ) ) != 0 )
+		{
+			s = strchr( c->text, '/' );
+			if( s )
+				*s = '\0';
+			jabber_set_me( ic, c->text );
+			imcb_log( ic, "Server claims your JID is `%s' instead of `%s'. "
+			          "This mismatch may cause problems with groupchats "
+			          "and possibly other things.",
+			          c->text, ic->acc->user );
+			if( s )
+				*s = '/';
+		}
+		else if( c && c->text_len && ( s = strchr( c->text, '/' ) ) &&
+		         strcmp( s + 1, set_getstr( &ic->acc->set, "resource" ) ) != 0 )
 			imcb_log( ic, "Server changed session resource string to `%s'", s + 1 );
 	}
 	
@@ -345,6 +393,8 @@ xt_status jabber_pkt_bind_sess( struct im_connection *ic, struct xt_node *node,
 	{
 		if( !jabber_get_roster( ic ) )
 			return XT_ABORT;
+		if( !jabber_iq_disco_server( ic ) )
+			return XT_ABORT;
 	}
 	
 	return XT_HANDLED;
@@ -793,3 +843,90 @@ xt_status jabber_iq_parse_server_features( struct im_connection *ic, struct xt_n
 
 	return XT_HANDLED;
 }
+
+static xt_status jabber_iq_version_response( struct im_connection *ic,
+	struct xt_node *node, struct xt_node *orig );
+
+void jabber_iq_version_send( struct im_connection *ic, struct jabber_buddy *bud, void *data )
+{
+	struct xt_node *node, *query;
+	
+	node = xt_new_node( "query", NULL, NULL );
+	xt_add_attr( node, "xmlns", XMLNS_VERSION );
+	query = jabber_make_packet( "iq", "get", bud->full_jid, node );
+	jabber_cache_add( ic, query, jabber_iq_version_response );
+
+	jabber_write_packet( ic, query );
+}
+
+static xt_status jabber_iq_version_response( struct im_connection *ic,
+	struct xt_node *node, struct xt_node *orig )
+{
+	struct xt_node *query;
+	GString *rets;
+	char *s;
+	char *ret[2] = {};
+	bee_user_t *bu;
+	struct jabber_buddy *bud = NULL;
+	
+	if( ( s = xt_find_attr( node, "from" ) ) &&
+	    ( bud = jabber_buddy_by_jid( ic, s, 0 ) ) &&
+	    ( query = xt_find_node( node->children, "query" ) ) &&
+	    ( bu = bee_user_by_handle( ic->bee, ic, bud->bare_jid ) ) )
+	{
+		rets = g_string_new( "Resource " );
+		g_string_append( rets, bud->resource );
+	}
+	else
+		return XT_HANDLED;
+	
+	for( query = query->children; query; query = query->next )
+		if( query->text_len > 0 )
+			g_string_append_printf( rets, " %s: %s,", query->name, query->text );
+	
+	g_string_truncate( rets, rets->len - 1 );
+	ret[0] = rets->str;
+	imcb_buddy_action_response( bu, "VERSION", ret, NULL );
+	g_string_free( rets, TRUE );
+	
+	return XT_HANDLED;
+}
+
+static xt_status jabber_iq_disco_server_response( struct im_connection *ic,
+	struct xt_node *node, struct xt_node *orig );
+
+static int jabber_iq_disco_server( struct im_connection *ic )
+{
+	struct xt_node *node, *iq;
+	struct jabber_data *jd = ic->proto_data;
+	
+	node = xt_new_node( "query", NULL, NULL );
+	xt_add_attr( node, "xmlns", XMLNS_DISCO_INFO );
+	iq = jabber_make_packet( "iq", "get", jd->server, node );
+	
+	jabber_cache_add( ic, iq, jabber_iq_disco_server_response );
+	return jabber_write_packet( ic, iq );
+}
+
+static xt_status jabber_iq_disco_server_response( struct im_connection *ic,
+	struct xt_node *node, struct xt_node *orig )
+{
+	struct jabber_data *jd = ic->proto_data;
+	struct xt_node *id;
+	
+	if( ( id = xt_find_path( node, "query/identity" ) ) )
+	{
+		char *cat, *type, *name;
+		
+		if( !( cat = xt_find_attr( id, "category" ) ) ||
+		    !( type = xt_find_attr( id, "type" ) ) ||
+		    !( name = xt_find_attr( id, "name" ) ) )
+			return XT_HANDLED;
+		
+		if( strcmp( cat, "server" ) == 0 && strcmp( type, "im" ) == 0 &&
+		    strstr( name, "Google" ) != NULL )
+			jd->flags |= JFLAG_GTALK;
+	}
+	
+	return XT_HANDLED;
+}
