@@ -46,6 +46,10 @@ typedef enum
 	                                   activates all XEP-85 related code. */
 	JFLAG_XMLCONSOLE = 64,          /* If the user added an xmlconsole buddy. */
 	JFLAG_STARTTLS_DONE = 128,      /* If a plaintext session was converted to TLS. */
+
+	JFLAG_GTALK =  0x100000,        /* Is Google Talk, as confirmed by iq discovery */
+
+	JFLAG_SASL_FB = 0x10000,        /* Trying Facebook authentication. */
 } jabber_flags_t;
 
 typedef enum
@@ -91,6 +95,10 @@ struct jabber_data
 	
 	char *username;		/* USERNAME@server */
 	char *server;		/* username@SERVER -=> server/domain, not hostname */
+	char *me;		/* bare jid */
+	
+	const struct oauth2_service *oauth2_service;
+	char *oauth2_access_token;
 	
 	/* After changing one of these two (or the priority setting), call
 	   presence_send_update() to inform the server about the changes. */
@@ -154,6 +162,7 @@ struct jabber_chat
 	char *name;
 	char *my_full_jid; /* Separate copy because of case sensitivity. */
 	struct jabber_buddy *me;
+	char *invite;
 };
 
 struct jabber_transfer
@@ -187,6 +196,7 @@ struct jabber_transfer
 };
 
 #define JABBER_XMLCONSOLE_HANDLE "xmlconsole"
+#define JABBER_OAUTH_HANDLE "jabber_oauth"
 
 /* Prefixes to use for packet IDs (mainly for IQ packets ATM). Usually the
    first one should be used, but when storing a packet in the cache, a
@@ -213,7 +223,8 @@ struct jabber_transfer
 /* Some supported extensions/legacy stuff */
 #define XMLNS_AUTH         "jabber:iq:auth"                                      /* XEP-0078 */
 #define XMLNS_VERSION      "jabber:iq:version"                                   /* XEP-0092 */
-#define XMLNS_TIME         "jabber:iq:time"                                      /* XEP-0090 */
+#define XMLNS_TIME_OLD     "jabber:iq:time"                                      /* XEP-0090 */
+#define XMLNS_TIME         "urn:xmpp:time"                                       /* XEP-0202 */
 #define XMLNS_PING         "urn:xmpp:ping"                                       /* XEP-0199 */
 #define XMLNS_VCARD        "vcard-temp"                                          /* XEP-0054 */
 #define XMLNS_DELAY        "jabber:x:delay"                                      /* XEP-0091 */
@@ -230,6 +241,9 @@ struct jabber_transfer
 #define XMLNS_BYTESTREAMS  "http://jabber.org/protocol/bytestreams"              /* XEP-0065 */
 #define XMLNS_IBB          "http://jabber.org/protocol/ibb"                      /* XEP-0047 */
 
+/* jabber.c */
+void jabber_connect( struct im_connection *ic );
+
 /* iq.c */
 xt_status jabber_pkt_iq( struct xt_node *node, gpointer data );
 int jabber_init_iq_auth( struct im_connection *ic );
@@ -240,6 +254,7 @@ int jabber_add_to_roster( struct im_connection *ic, const char *handle, const ch
 int jabber_remove_from_roster( struct im_connection *ic, char *handle );
 xt_status jabber_iq_query_features( struct im_connection *ic, char *bare_jid );
 xt_status jabber_iq_query_server( struct im_connection *ic, char *jid, char *xmlns );
+void jabber_iq_version_send( struct im_connection *ic, struct jabber_buddy *bud, void *data );
 
 /* si.c */
 int jabber_si_handle_request( struct im_connection *ic, struct xt_node *node, struct xt_node *sinode );
@@ -297,6 +312,7 @@ void jabber_buddy_remove_all( struct im_connection *ic );
 time_t jabber_get_timestamp( struct xt_node *xt );
 struct jabber_error *jabber_error_parse( struct xt_node *node, char *xmlns );
 void jabber_error_free( struct jabber_error *err );
+gboolean jabber_set_me( struct im_connection *ic, const char *me );
 
 extern const struct jabber_away_state jabber_away_state_list[];
 
@@ -304,7 +320,7 @@ extern const struct jabber_away_state jabber_away_state_list[];
 int jabber_write_packet( struct im_connection *ic, struct xt_node *node );
 int jabber_write( struct im_connection *ic, char *buf, int len );
 gboolean jabber_connected_plain( gpointer data, gint source, b_input_condition cond );
-gboolean jabber_connected_ssl( gpointer data, void *source, b_input_condition cond );
+gboolean jabber_connected_ssl( gpointer data, int returncode, void *source, b_input_condition cond );
 gboolean jabber_start_stream( struct im_connection *ic );
 void jabber_end_stream( struct im_connection *ic );
 
@@ -313,9 +329,17 @@ xt_status sasl_pkt_mechanisms( struct xt_node *node, gpointer data );
 xt_status sasl_pkt_challenge( struct xt_node *node, gpointer data );
 xt_status sasl_pkt_result( struct xt_node *node, gpointer data );
 gboolean sasl_supported( struct im_connection *ic );
+void sasl_oauth2_init( struct im_connection *ic );
+int sasl_oauth2_get_refresh_token( struct im_connection *ic, const char *msg );
+int sasl_oauth2_refresh( struct im_connection *ic, const char *refresh_token );
+
+extern const struct oauth2_service oauth2_service_google;
+extern const struct oauth2_service oauth2_service_facebook;
+extern const struct oauth2_service oauth2_service_mslive;
 
 /* conference.c */
 struct groupchat *jabber_chat_join( struct im_connection *ic, const char *room, const char *nick, const char *password );
+struct groupchat *jabber_chat_with( struct im_connection *ic, char *who );
 struct groupchat *jabber_chat_by_jid( struct im_connection *ic, const char *name );
 void jabber_chat_free( struct groupchat *c );
 int jabber_chat_msg( struct groupchat *ic, char *message, int flags );
