@@ -107,7 +107,10 @@ static void msn_logout( struct im_connection *ic )
 			md->groups = g_slist_remove( md->groups, mg );
 		}
 		
-		g_tree_destroy( md->domaintree );
+		g_free( md->profile_rid );
+		
+		if( md->domaintree )
+			g_tree_destroy( md->domaintree );
 		md->domaintree = NULL;
 		
 		while( md->grpq )
@@ -321,21 +324,15 @@ static char *set_eval_display_name( set_t *set, char *value )
 	struct im_connection *ic = acc->ic;
 	struct msn_data *md = ic->proto_data;
 	
-	if( strlen( value ) > 129 )
-	{
-		imcb_log( ic, "Maximum name length exceeded" );
-		return NULL;
-	}
-	
-	if( md->flags & MSN_GOT_PROFILE_DN )
-		imcb_log( ic, "Warning: Persistent name changes for this account have to be done "
-		              "in the profile. BitlBee doesn't currently support this." );
-	
 	if( md->flags & MSN_EMAIL_UNVERIFIED )
 		imcb_log( ic, "Warning: Your e-mail address is unverified. MSN doesn't allow "
 		              "changing your display name until your e-mail address is verified." );
 	
-	msn_soap_addressbook_set_display_name( ic, value );
+	if( md->flags & MSN_GOT_PROFILE_DN )
+		msn_soap_profile_set_dn( ic, value );
+	else
+		msn_soap_addressbook_set_display_name( ic, value );
+	
 	return msn_ns_set_display_name( ic, value ) ? value : NULL;
 }
 
@@ -357,12 +354,36 @@ static void msn_buddy_data_free( bee_user_t *bu )
 	g_tree_remove( md->domaintree, bu->handle );
 }
 
+GList *msn_buddy_action_list( bee_user_t *bu )
+{
+	static GList *ret = NULL;
+	
+	if( ret == NULL )
+	{
+		static const struct buddy_action ba[2] = {
+			{ "NUDGE", "Draw attention" },
+		};
+		
+		ret = g_list_prepend( ret, (void*) ba + 0 );
+	}
+	
+	return ret;
+}
+
+void *msn_buddy_action( struct bee_user *bu, const char *action, char * const args[], void *data )
+{
+	if( g_strcasecmp( action, "NUDGE" ) == 0 )
+		msn_buddy_msg( bu->ic, bu->handle, NUDGE_MESSAGE, 0 );
+	
+	return NULL;
+}
+
 void msn_initmodule()
 {
 	struct prpl *ret = g_new0(struct prpl, 1);
 	
 	ret->name = "msn";
-    ret->mms = 1409;         /* this guess taken from libotr UPGRADING file */
+	ret->mms = 1409;         /* this guess taken from libotr UPGRADING file */
 	ret->login = msn_login;
 	ret->init = msn_init;
 	ret->logout = msn_logout;
@@ -385,6 +406,8 @@ void msn_initmodule()
 	ret->handle_cmp = g_strcasecmp;
 	ret->buddy_data_add = msn_buddy_data_add;
 	ret->buddy_data_free = msn_buddy_data_free;
+	ret->buddy_action_list = msn_buddy_action_list;
+	ret->buddy_action = msn_buddy_action;
 	
 	//ret->transfer_request = msn_ftp_transfer_request;
 
