@@ -29,6 +29,7 @@
 /* Some hackish magicstrings to make special-purpose messages/switchboards.
  */
 #define TYPING_NOTIFICATION_MESSAGE "\r\r\rBEWARE, ME R TYPINK MESSAGE!!!!\r\r\r"
+#define NUDGE_MESSAGE "\r\r\rSHAKE THAT THING\r\r\r"
 #define GROUPCHAT_SWITCHBOARD_MESSAGE "\r\r\rME WANT TALK TO MANY PEOPLE\r\r\r"
 #define SB_KEEPALIVE_MESSAGE "\r\r\rDONT HANG UP ON ME!\r\r\r"
 
@@ -69,9 +70,15 @@
                            "TypingUser: %s\r\n" \
                            "\r\n\r\n"
 
-#define SB_KEEPALIVE_HEADERS "MIME-Version: 1.0\r\n" \
-                             "Content-Type: text/x-ping\r\n" \
-                             "\r\n\r\n"
+#define MSN_NUDGE_HEADERS "MIME-Version: 1.0\r\n" \
+			  "Content-Type: text/x-msnmsgr-datacast\r\n" \
+			  "\r\n" \
+			  "ID: 1\r\n" \
+			  "\r\n"
+
+#define MSN_SB_KEEPALIVE_HEADERS "MIME-Version: 1.0\r\n" \
+                                 "Content-Type: text/x-ping\r\n" \
+                                 "\r\n\r\n"
 
 #define PROFILE_URL "http://members.msn.com/"
 
@@ -118,6 +125,7 @@ struct msn_data
 	
 	const struct msn_away_state *away_state;
 	GSList *groups;
+	char *profile_rid;
 	
 	/* Mostly used for sending the ADL command; since MSNP13 the client
 	   is responsible for downloading the contact list and then sending
@@ -225,7 +233,6 @@ int msn_logged_in( struct im_connection *ic );
 int msn_buddy_list_add( struct im_connection *ic, msn_buddy_flags_t list, const char *who, const char *realname_, const char *group );
 int msn_buddy_list_remove( struct im_connection *ic, msn_buddy_flags_t list, const char *who, const char *group );
 void msn_buddy_ask( bee_user_t *bu );
-char *msn_findheader( char *text, char *header, int len );
 char **msn_linesplit( char *line );
 int msn_handler( struct msn_handler_data *h );
 void msn_msgq_purge( struct im_connection *ic, GSList **list );
