@@ -46,7 +46,7 @@ int msn_sb_write( struct msn_switchboard *sb, const char *fmt, ... )
 	va_end( params );
 	
 	if( getenv( "BITLBEE_DEBUG" ) )
-		fprintf( stderr, "->SB%d:%s", sb->fd, out );
+		fprintf( stderr, "->SB%d:%s\n", sb->fd, out );
 	
 	len = strlen( out );
 	st = write( sb->fd, out, len );
@@ -183,9 +183,14 @@ int msn_sb_sendmessage( struct msn_switchboard *sb, char *text )
 			buf = g_new0( char, i );
 			i = g_snprintf( buf, i, MSN_TYPING_HEADERS, sb->ic->acc->user );
 		}
+		else if( strcmp( text, NUDGE_MESSAGE ) == 0 )
+		{
+			buf = g_strdup( MSN_NUDGE_HEADERS );
+			i = strlen( buf );
+		}
 		else if( strcmp( text, SB_KEEPALIVE_MESSAGE ) == 0 )
 		{
-			buf = g_strdup( SB_KEEPALIVE_HEADERS );
+			buf = g_strdup( MSN_SB_KEEPALIVE_HEADERS );
 			i = strlen( buf );
 		}
 		else if( strncmp( text, MSN_INVITE_HEADERS, sizeof( MSN_INVITE_HEADERS ) - 1 ) == 0 ) 
@@ -302,7 +307,6 @@ gboolean msn_sb_connected( gpointer data, gint source, b_input_condition cond )
 {
 	struct msn_switchboard *sb = data;
 	struct im_connection *ic;
-	struct msn_data *md;
 	char buf[1024];
 	
 	/* Are we still alive? */
@@ -310,7 +314,6 @@ gboolean msn_sb_connected( gpointer data, gint source, b_input_condition cond )
 		return FALSE;
 	
 	ic = sb->ic;
-	md = ic->proto_data;
 	
 	if( source != sb->fd )
 	{
@@ -625,9 +628,12 @@ static int msn_sb_command( struct msn_handler_data *handler, char **cmd, int num
 		
 		/* If the person is offline, send an offline message instead,
 		   and don't report an error. */
+		/* TODO: Support for OIMs that works. (#874) */
+		/*
 		if( num == 217 )
 			msn_soap_oim_send_queue( ic, &sb->msgq );
 		else
+		*/
 			imcb_error( ic, "Error reported by switchboard server: %s", err->text );
 		
 		if( err->flags & STATUS_SB_FATAL )
@@ -669,20 +675,16 @@ static int msn_sb_message( struct msn_handler_data *handler, char *msg, int msgl
 	struct msn_switchboard *sb = handler->data;
 	struct im_connection *ic = sb->ic;
 	char *body;
-	int blen = 0;
 	
 	if( !num_parts )
 		return( 1 );
 	
 	if( ( body = strstr( msg, "\r\n\r\n" ) ) )
-	{
 		body += 4;
-		blen = msglen - ( body - msg );
-	}
 	
 	if( strcmp( cmd[0], "MSG" ) == 0 )
 	{
-		char *ct = msn_findheader( msg, "Content-Type:", msglen );
+		char *ct = get_rfc822_header( msg, "Content-Type:", msglen );
 		
 		if( !ct )
 			return( 1 );
@@ -711,8 +713,8 @@ static int msn_sb_message( struct msn_handler_data *handler, char *msg, int msgl
 		// Disable MSN ft support for now.
 		else if( g_strncasecmp( ct, "text/x-msmsgsinvite", 19 ) == 0 )
 		{
-			char *command = msn_findheader( body, "Invitation-Command:", blen );
-			char *cookie = msn_findheader( body, "Invitation-Cookie:", blen );
+			char *command = get_rfc822_header( body, "Invitation-Command:", blen );
+			char *cookie = get_rfc822_header( body, "Invitation-Cookie:", blen );
 			unsigned int icookie;
 			
 			g_free( ct );
@@ -750,7 +752,7 @@ static int msn_sb_message( struct msn_handler_data *handler, char *msg, int msgl
 		}
 		else if( g_strncasecmp( ct, "text/x-msmsgscontrol", 20 ) == 0 )
 		{
-			char *who = msn_findheader( msg, "TypingUser:", msglen );
+			char *who = get_rfc822_header( msg, "TypingUser:", msglen );
 			
 			if( who )
 			{
