@@ -387,10 +387,8 @@ int aim_get_command(aim_session_t *sess, aim_conn_t *conn)
 	 * or we break.  We must handle it just in case.
 	 */
 	if (aimbs_get8(&flaphdr) != 0x2a) {
-		guint8 start;
-
 		aim_bstream_rewind(&flaphdr);
-		start = aimbs_get8(&flaphdr);
+		aimbs_get8(&flaphdr);
 		imcb_error(sess->aux_data, "FLAP framing disrupted");
 		aim_conn_close(conn);
 		return -1;
