@@ -106,6 +106,16 @@ static void purple_init( account_t *acc )
 	g_string_printf( help, "BitlBee libpurple module %s (%s).\n\nSupported settings:",
 	                        (char*) acc->prpl->name, prpl->info->name );
 	
+	if( pi->user_splits )
+	{
+		GList *l;
+		g_string_append_printf( help, "\n* username: Username" );
+		for( l = pi->user_splits; l; l = l->next )
+			g_string_append_printf( help, "%c%s",
+			                        purple_account_user_split_get_separator( l->data ),
+			                        purple_account_user_split_get_text( l->data ) );
+	}
+	
 	/* Convert all protocol_options into per-account setting variables. */
 	for( i = pi->protocol_options; i; i = i->next )
 	{
@@ -180,9 +190,11 @@ static void purple_init( account_t *acc )
 		default:
 			/** No way to talk to the user right now, invent one when
 			this becomes important.
-			irc_usermsg( acc->irc, "Setting with unknown type: %s (%d) Expect stuff to break..\n",
+			irc_rootmsg( acc->irc, "Setting with unknown type: %s (%d) Expect stuff to break..\n",
 			             name, purple_account_option_get_type( o ) );
 			*/
+			g_string_append_printf( help, "\n* [%s] UNSUPPORTED (type %d)",
+			                        name, purple_account_option_get_type( o ) );
 			name = NULL;
 		}
 		
@@ -749,6 +761,8 @@ static void prplcb_blist_update( PurpleBuddyList *list, PurpleBlistNode *node )
 		
 		if( bud->server_alias )
 			imcb_rename_buddy( ic, bud->name, bud->server_alias );
+		else if( bud->alias )
+			imcb_rename_buddy( ic, bud->name, bud->alias );
 		
 		if( group )
 			imcb_add_buddy( ic, bud->name, purple_group_get_name( group ) );
