@@ -378,6 +378,7 @@ void byahoo_initmodule( )
 {
 	struct prpl *ret = g_new0(struct prpl, 1);
 	ret->name = "yahoo";
+    ret->mms = 832;           /* this guess taken from libotr UPGRADING file */
 	ret->init = byahoo_init;
 	
 	ret->login = byahoo_login;
@@ -588,6 +589,8 @@ void ext_yahoo_status_changed( int id, const char *who, int stat, const char *ms
 	
 	if( away )
 		flags |= OPT_AWAY;
+	if( mobile )
+		flags |= OPT_MOBILE;
 	
 	switch (stat)
 	{
