@@ -2,6 +2,7 @@
 #include <glib.h>
 #include <gmodule.h>
 #include <check.h>
+#include <locale.h>
 #include "bitlbee.h"
 #include "testsuite.h"
 
@@ -59,9 +60,6 @@ Suite *help_suite(void);
 /* From check_user.c */
 Suite *user_suite(void);
 
-/* From check_crypting.c */
-Suite *crypting_suite(void);
-
 /* From check_set.c */
 Suite *set_suite(void);
 
@@ -94,6 +92,8 @@ int main (int argc, char **argv)
 	g_option_context_free(pc);
 
 	log_init();
+	b_main_init();
+	setlocale(LC_CTYPE, "");
 
 	if (verbose) {
 		log_link( LOGLVL_ERROR, LOGOUTPUT_CONSOLE );
@@ -114,7 +114,6 @@ int main (int argc, char **argv)
 	srunner_add_suite(sr, irc_suite());
 	srunner_add_suite(sr, help_suite());
 	srunner_add_suite(sr, user_suite());
-	srunner_add_suite(sr, crypting_suite());
 	srunner_add_suite(sr, set_suite());
 	srunner_add_suite(sr, jabber_sasl_suite());
 	srunner_add_suite(sr, jabber_util_suite());
